﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.StorageClient;
using System.Data.Services.Client;

namespace StorageClientLibrary17
{
    class Program
    {
        private static string accountName = "<your storage account name>";
        private static string accountKey = "<Base64 encoded storage account key>";
        private static CloudStorageAccount storageAccount;
        private static string tableName = "Customer";
        private static CloudTableClient cloudTableClient;

        static void Main(string[] args)
        {
            storageAccount = new CloudStorageAccount(new StorageCredentialsAccountAndKey(accountName, accountKey), true);
            cloudTableClient = storageAccount.CreateCloudTableClient();
            CreateTable();
            Console.WriteLine("Table created successfully");
            FetchEntities();
            Console.WriteLine("Entity merged successfully");
            DeleteTable();
            Console.WriteLine("Table deleted successfully");
            Console.WriteLine("Press any key to terminate");
            Console.ReadLine();
        }

        static void CreateTable()
        {
            cloudTableClient.CreateTableIfNotExist(tableName);
        }

        static void DeleteTable()
        {
            cloudTableClient.DeleteTableIfExist(tableName);
        }

        static CustomerEntity CreateEntity()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };

            var serviceContext = cloudTableClient.GetDataServiceContext();
            serviceContext.AddObject(tableName, customer);
            serviceContext.SaveChangesWithRetries();
            return customer;
        }

        static void DeleteEntity()
        {
            var customer = CreateEntity();
            var serviceContext = cloudTableClient.GetDataServiceContext();
            serviceContext.AttachTo(tableName, customer, "*");
            serviceContext.DeleteObject(customer);
            serviceContext.SaveChangesWithRetries();
        }

        static void ReplaceEntity()
        {
            var customer = CreateEntity();
            customer.LastOrderDate = null;
            var serviceContext = cloudTableClient.GetDataServiceContext();
            serviceContext.AttachTo(tableName, customer, "*");
            serviceContext.UpdateObject(customer);
            serviceContext.SaveChangesWithRetries(SaveChangesOptions.ReplaceOnUpdate);
        }

        static void MergeEntity()
        {
            var customer = CreateEntity();
            customer.LastOrderDate = null;
            var serviceContext = cloudTableClient.GetDataServiceContext();
            serviceContext.AttachTo(tableName, customer, "*");
            serviceContext.UpdateObject(customer);
            serviceContext.SaveChangesWithRetries();
        }

        static void InsertOrReplaceEntity()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };
            var serviceContext = cloudTableClient.GetDataServiceContext();
            serviceContext.AttachTo(tableName, customer, null);
            serviceContext.UpdateObject(customer);
            serviceContext.SaveChangesWithRetries(SaveChangesOptions.ReplaceOnUpdate);
        }

        static void InsertOrMergeEntity()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };
            var serviceContext = cloudTableClient.GetDataServiceContext();
            serviceContext.AttachTo(tableName, customer, null);
            serviceContext.UpdateObject(customer);
            serviceContext.SaveChangesWithRetries();
        }

        static void InsertEntityBatchOperation()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };
            var serviceContext = cloudTableClient.GetDataServiceContext();
            serviceContext.AddObject(tableName, customer);
            customer = new CustomerEntity()
            {
                FirstName = "Jane",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-5)
            };
            serviceContext.AddObject(tableName, customer);
            customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Doe",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-7)
            };
            serviceContext.AddObject(tableName, customer);
            customer = new CustomerEntity()
            {
                FirstName = "Jane",
                LastName = "Doe",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-3)
            };
            serviceContext.AddObject(tableName, customer);
            serviceContext.SaveChangesWithRetries(SaveChangesOptions.Batch);
        }

        static void FetchSingleEntity()
        {
            var customer = CreateEntity();
            string partitionKey = customer.PartitionKey;
            string rowKey = customer.RowKey;
            CloudStorageAccount storageAccount = new CloudStorageAccount(new StorageCredentialsAccountAndKey(accountName, accountKey), true);
            CloudTableClient cloudTableClient = storageAccount.CreateCloudTableClient();
            var serviceContext = cloudTableClient.GetDataServiceContext();
            IQueryable<CustomerEntity> customerQuery = (from entity in serviceContext.CreateQuery<CustomerEntity>(tableName) where entity.PartitionKey == partitionKey && entity.RowKey == rowKey select entity);
            var returnedCustomer = customerQuery.FirstOrDefault();
        }

        static void FetchEntities()
        {
            InsertEntityBatchOperation();
            CloudStorageAccount storageAccount = new CloudStorageAccount(new StorageCredentialsAccountAndKey(accountName, accountKey), true);
            CloudTableClient cloudTableClient = storageAccount.CreateCloudTableClient();
            var serviceContext = cloudTableClient.GetDataServiceContext();
            //Specify the filter condition on LastName == 'Smith'
            IQueryable<CustomerEntity> customerQuery = (from entity in serviceContext.CreateQuery<CustomerEntity>(tableName) where entity.LastName == "Smith" select entity);
            var result = customerQuery.ToList();
        }
    }

    public class CustomerEntity : TableServiceEntity
    {
        public CustomerEntity()
        {
            PartitionKey = "Customer";
            RowKey = Guid.NewGuid().ToString();
        }

        public string FirstName
        {
            get;
            set;
        }

        public string LastName
        {
            get;
            set;
        }

        public DateTime? LastOrderDate
        {
            get;
            set;
        }
    }

}
