﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.WindowsAzure;
using Microsoft.WindowsAzure.Storage;
using Microsoft.WindowsAzure.Storage.Auth;
using Microsoft.WindowsAzure.Storage.Table;

namespace StorageClientLibrary20
{
    class Program
    {
        private static string accountName = "<your storage account name>";
        private static string accountKey = "<Base64 encoded storage account key>";
        private static CloudStorageAccount storageAccount;
        private static string tableName = "Customer";
        private static CloudTableClient cloudTableClient;

        static void Main(string[] args)
        {
            storageAccount = new CloudStorageAccount(new StorageCredentials(accountName, accountKey), true);
            cloudTableClient = storageAccount.CreateCloudTableClient();
            CreateTable();
            Console.WriteLine("Table created successfully");
            FetchEntities();
            Console.WriteLine("Entity replaced successfully");
            DeleteTable();
            Console.WriteLine("Table deleted successfully");
            Console.WriteLine("Press any key to terminate");
            Console.ReadLine();
        }

        static void CreateTable()
        {
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            table.CreateIfNotExists();
        }

        static void DeleteTable()
        {
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            table.DeleteIfExists();
        }

        static CustomerEntity CreateEntity()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };
            
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableOperation insertOperation = TableOperation.Insert(customer);
            table.Execute(insertOperation);
            return customer;
        }

        static void DeleteEntity()
        {
            var customer = CreateEntity();
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableOperation deleteOperation = TableOperation.Delete(customer);
            table.Execute(deleteOperation);
        }

        static void ReplaceEntity()
        {
            var customer = CreateEntity();
            customer.LastOrderDate = null;
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableOperation replaceOperation = TableOperation.Replace(customer);
            table.Execute(replaceOperation);
        }

        static void MergeEntity()
        {
            var customer = CreateEntity();
            customer.LastOrderDate = DateTime.UtcNow;
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableOperation mergeOperation = TableOperation.Merge(customer);
            table.Execute(mergeOperation);
        }

        static void InsertOrReplaceEntity()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableOperation insertOrReplaceOperation = TableOperation.InsertOrReplace(customer);
            table.Execute(insertOrReplaceOperation);
        }

        static void InsertOrMergeEntity()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableOperation insertOrMergeOperation = TableOperation.InsertOrMerge(customer);
            table.Execute(insertOrMergeOperation);
        }

        static void InsertEntityBatchOperation()
        {
            var customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-10)
            };
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableBatchOperation batchOperation = new TableBatchOperation();
            batchOperation.Insert(customer);
            customer = new CustomerEntity()
            {
                FirstName = "Jane",
                LastName = "Smith",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-5)
            };
            batchOperation.Insert(customer);
            customer = new CustomerEntity()
            {
                FirstName = "John",
                LastName = "Doe",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-7)
            };
            batchOperation.Insert(customer);
            customer = new CustomerEntity()
            {
                FirstName = "Jane",
                LastName = "Doe",
                LastOrderDate = DateTime.UtcNow.Date.AddDays(-3)
            };
            batchOperation.Insert(customer);
            table.ExecuteBatch(batchOperation);
        }

        static void FetchSingleEntity()
        {
            var customer = CreateEntity();
            string partitionKey = customer.PartitionKey;
            string rowKey = customer.RowKey;
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            TableOperation retrieveOperation = TableOperation.Retrieve<CustomerEntity>(partitionKey, rowKey);
            TableResult retrievedResult = table.Execute(retrieveOperation);
            CustomerEntity fetchedCustomer = retrievedResult.Result as CustomerEntity;
        }

        static void FetchEntities()
        {
            InsertEntityBatchOperation();
            CloudStorageAccount storageAccount = new CloudStorageAccount(new StorageCredentials(accountName, accountKey), true);
            CloudTableClient cloudTableClient = storageAccount.CreateCloudTableClient();
            CloudTable table = cloudTableClient.GetTableReference(tableName);
            //Specify the filter condition on LastName == 'Smith'
            TableQuery<CustomerEntity> query = (new TableQuery<CustomerEntity>()).Where(TableQuery.GenerateFilterCondition("LastName", QueryComparisons.Equal, "Smith"));
            var result = table.ExecuteQuery<CustomerEntity>(query);
        }

    }

    public class CustomerEntity : TableEntity
    {
        public CustomerEntity()
        {
            PartitionKey = "Customer";
            RowKey = Guid.NewGuid().ToString();
        }

        public string FirstName
        {
            get;
            set;
        }

        public string LastName
        {
            get;
            set;
        }

        public DateTime? LastOrderDate
        {
            get;
            set;
        }
    }
}
